import math,json,sys
import lib_pcb

#
# uncomment for desired output:
#
output = "top, labels, and exterior"
# output = "top, labels, holes, and exterior"
# output = "top, bottom, labels, and exterior"
# output = "top, bottom, labels, holes, and exterior"
# output = "top traces"
# output = "top traces and exterior"
# output = "bottom traces reversed"
# output = "bottom traces reversed and exterior"
# output = "holes"
# output = "interior"
# output = "holes and interior"
# output = "exterior"
# output = "solder mask"

#
# definitions
#
wire = lib_pcb.wire
chamfer = lib_pcb.chamfer
point = lib_pcb.point
via = lib_pcb.via
color = lib_pcb.color

Red = lib_pcb.Red
Green = lib_pcb.Green
Blue = lib_pcb.Blue
Gray = lib_pcb.Gray
White = lib_pcb.White
Teal = lib_pcb.Teal
Pink = lib_pcb.Pink
Yellow = lib_pcb.Yellow
Brown = lib_pcb.Brown
Navy = lib_pcb.Navy
Tan = lib_pcb.Tan

#
# define board
#
width = 1 # board width
height = 1.43 # board height
x = 1 # x origin
y = 1 # y origin
zt = 0 # top z
zb = -0.06 # bottom z
w = .015 # wire width
rv = 0.016 # via size
rp = 0.03 # pad size
mask = .004 # solder mask size

pcb = lib_pcb.PCB(x, y, width, height, mask)

USB = lib_pcb.USB_A_plug(zb, zt)
USB.add(pcb, x+width/2, y+height-0.28, angle=90)

IC1 = lib_pcb.SAMD11C('IC1\nd11c')
pcb = IC1.add(pcb, x+width/2+0.025, y+height/2+0.12)

J2 = lib_pcb.header_FTDI('J2 FTDI')
pcb = J2.add(pcb, IC1.x-0.32, IC1.y-0.075, angle=180)

R1 = lib_pcb.R_1206('R1\n1.2k')
pcb = R1.add(pcb, J2.pad[5].x+0.12, J2.pad[5].y+0.03, angle=-90)

C1 = lib_pcb.C_1206("C1\n1uF")
pcb = C1.add(pcb, x+width-0.16, y+height-0.45)

switch = lib_pcb.header_3()
pcb = switch.add(pcb, x+width-0.16, y+0.2)

v_reg = lib_pcb.regulator_SOT23("SOT23")
pcb = v_reg.add(pcb, x+width-0.12, y+height-0.6, angle=90)

C2 = lib_pcb.C_1206("C2\n1uF")
pcb = C2.add(pcb, x+0.1, y+0.37)

updi = lib_pcb.header_PDI()
pcb = updi.add(pcb, x+0.3, y+0.2)

jtag = lib_pcb.header_4()
pcb = jtag.add(pcb, x+0.55, y+0.55)

R2 = lib_pcb.R_1206('R2\n4.7k')
pcb = R2.add(pcb, x+0.53, y+0.14, angle=90)

jumper = lib_pcb.header_2()
pcb = jumper.add(pcb, x+0.65, y+0.25)

# USB DATA

pcb = wire(pcb, w, USB.pad[2],
           point(USB.pad[2].x, USB.pad[2].y-0.08),
           point(IC1.pad[1].x+0.06, IC1.pad[1].y+0.05),
           chamfer(IC1.pad[1].x+0.06, IC1.pad[9].y, dist=0.06),
           IC1.pad[9])

pcb = wire(pcb, w, USB.pad[3],
           point(USB.pad[3].x, USB.pad[3].y-0.07),
           point(USB.pad[3].x-0.026, USB.pad[2].y-0.09),
           chamfer(USB.pad[3].x-0.026, IC1.pad[10].y, dist=0.04),
           IC1.pad[10])

pcb = wire(pcb, w, USB.pad[4],
           point(USB.pad[4].x, USB.pad[4].y-0.07),
           chamfer(USB.pad[4].x-0.09, USB.pad[4].y-0.07, dist=0.04),
           chamfer(USB.pad[4].x-0.09, IC1.pad[11].y, dist=0.02),
           IC1.pad[11])

pcb = wire(pcb, w,
           point(USB.pad[4].x, USB.pad[4].y-0.07),
           chamfer(v_reg.pad[3].x, USB.pad[4].y-0.07, dist=0.08),
           v_reg.pad[3])

pcb = wire(pcb, w, IC1.pad[12],
           chamfer(IC1.pad[12].x-0.055, IC1.pad[12].y, dist=0.02),
           chamfer(IC1.pad[12].x-0.055, IC1.pad[14].y+0.055, dist=0.02),
           point(C1.pad[1].x-0.02, IC1.pad[14].y+0.055),
           C1.pad[1])

pcb = wire(pcb, w, jtag.pad[1], IC1.pad[7])
pcb = wire(pcb, w, jtag.pad[2], IC1.pad[8])
pcb = wire(pcb, w, jtag.pad[3],
           point(jtag.pad[3].x+0.08, jtag.pad[3].y+0.05),
           chamfer(jtag.pad[3].x+0.08, IC1.pad[6].y, dist=0.06),
           IC1.pad[6])

# V_REG

pcb = wire(pcb, w, C1.pad[1],
           chamfer(v_reg.pad[1].x, C1.pad[1].y, dist=0.05),
           v_reg.pad[1])

pcb = wire(pcb, w,
           point(jumper.pad[0].x+0.1, switch.pad[1].y*0.5+switch.pad[2].y*0.5),
           chamfer(jumper.pad[0].x+0.1, jtag.pad[4].y, dist=0.04),
           jtag.pad[4])

pcb = wire(pcb, w, v_reg.pad[1], switch.pad[2])

pcb = wire(pcb, w, v_reg.pad[2],
           point(v_reg.pad[2].x, y+0.57),
           point(v_reg.pad[2].x+0.04, y+0.53),
           chamfer(v_reg.pad[2].x+0.04, switch.pad[3].y, dist=0.06),
           switch.pad[3]
           )

# SWITCH

pcb = wire(pcb, w, switch.pad[1],
           point(switch.pad[1].x-0.1, updi.pad[1].y*0.5+updi.pad[3].y*0.5),
           chamfer(updi.pad[2].x, updi.pad[1].y*0.5+updi.pad[3].y*0.5, dist=0.02),
           updi.pad[2])

pcb = wire(pcb, w, switch.pad[3],
           point(switch.pad[3].x-0.2, y+0.015),
           chamfer(C2.pad[1].x, y+0.015, dist=0.06),
           C2.pad[1])

pcb = wire(pcb, w, updi.pad[1], R2.pad[1])
pcb = wire(pcb, w, R2.pad[2], jumper.pad[2])

pcb = wire(pcb, w, R1.pad[1], IC1.pad[1])

pcb = wire(pcb, w, C2.pad[1],
           chamfer(C2.pad[1].x, J2.pad[6].y+0.055, dist=0.04),
           chamfer(USB.pad[1].x, J2.pad[6].y+0.055, dist=0.02),
           USB.pad[1])

# TX
pcb = wire(pcb, w, jumper.pad[1],
           chamfer(jumper.pad[1].x-0.07, jumper.pad[1].y, dist=0.02),
           chamfer(jumper.pad[1].x-0.07, jtag.pad[0].y),
           chamfer(jtag.pad[2].x+0.08, jtag.pad[0].y),
           point(jtag.pad[2].x+0.08, IC1.pad[14].y-0.1),
           point(IC1.pad[14].x+0.07, IC1.pad[14].y-0.05),
           chamfer(IC1.pad[14].x+0.07, IC1.pad[14].y, dist=0.015),
           IC1.pad[14])

# RX
pcb = wire(pcb, w, R1.pad[2], J2.pad[5])

pcb = wire(pcb, w, updi.pad[2],
           chamfer(updi.pad[2].x-0.075, updi.pad[2].y, dist=0.015),
           chamfer(updi.pad[2].x-0.075, J2.pad[3].y, dist=0.02),
           J2.pad[3])

pcb = wire(pcb, w, updi.pad[1],
           point(updi.pad[1].x-0.08, updi.pad[1].y-0.05),
           chamfer(updi.pad[2].x-0.11, updi.pad[1].y-0.05, dist=0.04),
           chamfer(updi.pad[2].x-0.11, J2.pad[5].y, dist=0.03),
           J2.pad[5])

pcb = wire(pcb, w, C2.pad[2], updi.pad[6])

pcb = wire(pcb, w, v_reg.pad[3],
           point(v_reg.pad[3].x, y+0.55),
           point(switch.pad[0].x+0.08, y+0.5),
           point(switch.pad[1].x+0.08, switch.pad[1].y*0.5+switch.pad[2].y*0.5),
           chamfer(R1.pad[2].x, switch.pad[1].y*0.5+switch.pad[2].y*0.5, dist=0.02),
           chamfer(R1.pad[2].x, J2.pad[4].y, dist=0.04),
           J2.pad[4])


active_board = ("top" in output) or ("bottom" in output)
active_label = "labels" in output
active_exterior = "exterior" in output
active_holes = "holes" in output

outputs = {}
f_board = {"f": pcb.board, "color": Tan, "alpha": 1.0, "active": active_board}
f_labels1 = {"f": pcb.labels1, "color": 0x8dff6a, "alpha": 0.8, "active": active_label}
f_labels2 = {"f": pcb.labels2, "color": 0xff0d0d, "alpha": 0.8, "active": active_label}
f_exterior = {"f": pcb.exterior, "color": White, "alpha": 1.0, "active": active_exterior}
f_holes = {"f": pcb.holes, "color": Blue, "alpha": 0.8, "active": active_holes}
outputs["functions"] = [f_board, f_labels1, f_labels2, f_exterior, f_holes]

if ("bottom" in output) and ("top" in output):
    outputs["layers"] = [zt, zb]
elif "bottom" in output:
    outputs["layers"] = [zb]
else:
    outputs["layers"] = [zt]

#
# set limits and parameters
#
border = 0.05
outputs["xmin"] = x-border # min x to render
outputs["xmax"] = x+width+border # max x to render
outputs["ymin"] = y-border # min y to render
outputs["ymax"] = y+height+border # max y to render
outputs["mm_per_unit"] = 25.4 # use inch units
outputs["type"] = "RGB" # use RGB color

#
# send output
#
json.dump(outputs, sys.stdout)
